// UDA'Software 2003
// File: removecolors.cpp

/*

  Copyright (C) 1988 Free Software Foundation, Inc.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Questo script di esempio in C++ serve per rimuovere i codici di
 * colore, grassetto, sottolineatura, ecc... nei messaggi provenienti
 * dagli altri utenti.
 *
 * Si abilita/disabilita digitando il comando:
 *
 * /removecolors
 *
 * alla riga dei comandi di X-Chat 2.
 *
 */

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Include la libreria C++ per la gestione degli X-Chat Plug-in.
 *
 */
#include "plugin.h"

#include <ctype.h>

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Dichiara a livello globale l'Oggetto Plugin.
 *
 * L'oggetto ingloba tutte le Proprieta' e le Azioni necessarie per la
 * gestione di un plug-in.
 *
 */
static Plugin		PluginObj;

/*
 * Dichiara e inizializza una variabile flag che ha il compito di
 * determinare se il removing deve funzionare o meno; per default e'
 * attivo (1).
 *
 */
static int		enable = 1;

/*
 * Dichiara ed inizializza un vettore di stringhe contenente gli
 * eventi che la funzione RemoveColorAndEmitEvent() deve gestire.
 *
 */
const char	*cpsaEvent[] = {
  "Channel Action", 
  "Channel Action Hilight", 
  "Channel Message", 
  "Channel Msg Hilight", 
  "Channel Notice", 
  "Kick", 
  "Killed", 
  "Notice", 
  "Part with Reason", 
  "Private Message", 
  "Private Message to Dialog", 
  "Quit", 
  "Receive Wallops", 
  "Server Error", 
  "Server Notice", 
  "Server Text", 
  "Topic", 
  "Topic Change", 
  "WhoIs Away Line", 
  "You Kicked", 
  "DCC Chat Text"
};

/*
 * Definisce un enumerazione che servira' a stabilire l'evento che la
 * RemoveColorAndEmitEvent().
 *
 */
enum enum_event {
  ChannelAction, 
  ChannelActionHilight, 
  ChannelMessage, 
  ChannelMsgHilight, 
  ChannelNotice, 
  Kick, 
  Killed, 
  Notice, 
  PartwithReason, 
  PrivateMessage, 
  PrivateMessagetoDialog, 
  Quit, 
  ReceiveWallops, 
  ServerError, 
  ServerNotice, 
  ServerText, 
  Topic, 
  TopicChange, 
  WhoIsAwayLine, 
  YouKicked, 
  DCCChatText
};

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * La funzione RemoveColorAndEmitEvent() ha il compito di eliminare i
 * codici colore dal messaggio (passato come parametro).
 *
 */
static int RemoveColorAndEmitEvent(enum_event eEvent,		// Callback che ha invocato la funzione.
				   char *word[],		// Array dei parametri
				   int iMsgPos) {		// Posizione del testo da modificare.
  static int	bDonTEatNextEvent;

  if (!bDonTEatNextEvent) {
    if (enable && 
	word[iMsgPos][0] != '\0') {
      // Evita che l'evento successivo generato venga processato, dato
      // che si trattera' dell'evento creato ad hoc.
      bDonTEatNextEvent = 1;

      // Verifica la dimensione del testo da modificare e alloca la
      // stringa che dovra' contenere il nuovo messaggio.
      int	iStringDim = strlen(word[iMsgPos]);
      char	*psString = new char[iStringDim + 1];
      int	i, k;
      
      memset(psString, '\0', iStringDim);
      // Inizia il ciclo iterativo di copia dei caratteri; i caratteri
      // vengono copiati eliminando tutti i codici non ascii.
      for (i = 0, k = 0; i < iStringDim; i++, k++) {
	if (isprint(word[iMsgPos][i]))
	  psString[k] = word[iMsgPos][i];
	else {
	  --k;
	  // Ora deve andare alla ricerca della fine del codice
	  // numerico.
	  if (i + 5 < iStringDim && 
	      (isdigit(word[iMsgPos][i + 1]) && 
	       isdigit(word[iMsgPos][i + 2]) && 
	       word[iMsgPos][i + 3] == ',' && 
	       isdigit(word[iMsgPos][i + 4]) && 
	       isdigit(word[iMsgPos][i + 5]))) {		// Cxx,xx
	    i += 5;
	  } else if (i + 4 < iStringDim && 
		     (isdigit(word[iMsgPos][i + 1]) && 
		      isdigit(word[iMsgPos][i + 2]) && 
		      word[iMsgPos][i + 3] == ',' && 
		      isdigit(word[iMsgPos][i + 4]))) {		// Cxx,x
	    i += 4;
	  } else if (i + 4 < iStringDim && 
		     (isdigit(word[iMsgPos][i + 1]) && 
		      word[iMsgPos][i + 2] == ',' && 
		      isdigit(word[iMsgPos][i + 3]) && 
		      isdigit(word[iMsgPos][i + 4]))) {		// Cx,xx
	    i += 4;
	  } else if (i + 3 < iStringDim && 
		     (isdigit(word[iMsgPos][i + 1]) && 
		      word[iMsgPos][i + 2] == ',' && 
		      isdigit(word[iMsgPos][i + 3]))) {		// Cx,x
	    i += 3;
	  } else if (i + 3 < iStringDim && 
		     (isdigit(word[iMsgPos][i + 1]) && 
		      isdigit(word[iMsgPos][i + 2]))) {		// Cxx
	    i += 2;
	  } else if (i + 2 < iStringDim && 
		     (isdigit(word[iMsgPos][i + 1]) && 
		      !isdigit(word[iMsgPos][i + 2]))) {	// Cx
	    i += 1;
	  }
	}
      }
      
      // Termina la stringa.
      psString[k] = '\0';
      
      /*
       * A seconda del messaggio da generare, si comporta
       * opportunamente.
       *
       */
      switch (eEvent) {
      case ChannelAction:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case ChannelActionHilight:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case ChannelMessage:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, word[3], NULL);
	break;
      case ChannelMsgHilight:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case ChannelNotice:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], word[2], psString, NULL);
	break;
      case Kick:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], word[2], word[3], psString);
	break;
      case Killed:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case Notice:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case PartwithReason:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], word[2], word[3], psString);
	break;
      case PrivateMessage:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case PrivateMessagetoDialog:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case Quit:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, word[3], NULL);
	break;
      case ReceiveWallops:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case ServerError:
	PluginObj.emit_print(cpsaEvent[eEvent], psString, NULL, NULL, NULL);
	break;
      case ServerNotice:
	PluginObj.emit_print(cpsaEvent[eEvent], psString, word[2], NULL, NULL);
	break;
      case ServerText:
	PluginObj.emit_print(cpsaEvent[eEvent], psString, word[2], NULL, NULL);
	break;
      case Topic:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case TopicChange:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, word[3], NULL);
	break;
      case WhoIsAwayLine:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], psString, NULL, NULL);
	break;
      case YouKicked:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], word[2], word[3], psString);
	break;
      case DCCChatText:
	PluginObj.emit_print(cpsaEvent[eEvent], word[1], word[2], word[3], psString);
	break;
      default:
	PluginObj.print("---\002 RemoveColors Plug-in:\0034 Internal Error");
      };

      return XCHAT_EAT_ALL;
    }
  } else
    bDonTEatNextEvent = 0;
  
  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * DA QUI IN POI SONO DEFINITE LE CALLBACK INVOCATE IN BASE AGLI
 * EVENTI "NORMALI".
 *
 */
// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelAction(char *word[], void *userdata) {
  // *$1 $2
  // $1 Nickname
  // $2 The action

  return RemoveColorAndEmitEvent(ChannelAction, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelActionHilight(char *word[], void *userdata) {
  // *$1 $2
  // $1 Nickname
  // $2 The action

  return RemoveColorAndEmitEvent(ChannelActionHilight, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelMessage(char *word[], void *userdata) {
  // <$1>$2
  // $1 Nickname
  // $2 The text
  // $3 Mode char

  return RemoveColorAndEmitEvent(ChannelMessage, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelMsgHilight(char *word[], void *userdata) {
  // <$1>$2

  return RemoveColorAndEmitEvent(ChannelMsgHilight, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelNotice(char *word[], void *userdata) {
  // -$1/$2-$3
  // $1 Who it's from
  // $2 The Channel it's going to
  // $3 The message

  return RemoveColorAndEmitEvent(ChannelNotice, word, 3);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Kick(char *word[], void *userdata) {
  // <--$1 has kicked $2 from $3 ($4)
  // $1 The nickname of the kicker
  // $2 The person being kicked
  // $3 The channel
  // $4 The reason
  
  return RemoveColorAndEmitEvent(Kick, word, 4);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Killed(char *word[], void *userdata) {
  // You have been killed by $1 ($2)
  // $1 Nickname
  // $2 Reason

  return RemoveColorAndEmitEvent(Killed, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Notice(char *word[], void *userdata) {
  // -$1-$2
  // $1 Who it's from
  // $2 The message

  return RemoveColorAndEmitEvent(Notice, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_PartwithReason(char *word[], void *userdata) {
  // <--$1 ($2) has left $3 ($4)
  // $1 The nick of the person leaving
  // $2 The host of the person
  // $3 The channel
  // $4 The reason
  
  return RemoveColorAndEmitEvent(PartwithReason, word, 4);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_PrivateMessage(char *word[], void *userdata) {
  // *$1*$2
  // $1 Nickname
  // $2 The message

  return RemoveColorAndEmitEvent(PrivateMessage, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_PrivateMessagetoDialog(char *word[], void *userdata) {
  // <$1>$2
  // $1 Nickname
  // $2 The message

  return RemoveColorAndEmitEvent(PrivateMessagetoDialog, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Quit(char *word[], void *userdata) {
  // <--$1 has quit ($2)
  // $1 Nick
  // $2 Reason
  // $3 Host

  return RemoveColorAndEmitEvent(Quit, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ReceiveWallops(char *word[], void *userdata) {
  // -$1/Wallops-$2
  // $1 Nickname
  // $2 The message

  return RemoveColorAndEmitEvent(ReceiveWallops, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ServerError(char *word[], void *userdata) {
  // $1
  // $1 Text

  return RemoveColorAndEmitEvent(ServerError, word, 1);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ServerNotice(char *word[], void *userdata) {
  // $1
  // $1 Text
  // $2 Server Name

  return RemoveColorAndEmitEvent(ServerNotice, word, 1);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ServerText(char *word[], void *userdata) {
  // $1
  // $1 Text
  // $2 Server Name

  return RemoveColorAndEmitEvent(ServerText, word, 1);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Topic(char *word[], void *userdata) {
  // Topic for $1 is $2
  // $1 Channel
  // $2 Topic

  return RemoveColorAndEmitEvent(Topic, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_TopicChange(char *word[], void *userdata) {
  // $1 has changed the topic to: $2
  // $1 Nick of person who changed the topic
  // $2 Topic
  // $3 Channel

  return RemoveColorAndEmitEvent(TopicChange, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_WhoIsAwayLine(char *word[], void *userdata) {
  // [$1] is away ($2)
  // $1 Nickname
  // $2 Away reason

  return RemoveColorAndEmitEvent(WhoIsAwayLine, word, 2);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YouKicked(char *word[], void *userdata) {
  // You have been kicked from $2 by $3 ($4)
  // $1 The person being kicked
  // $2 The channel
  // $3 The nickname of the kicker
  // $4 The reason

  return RemoveColorAndEmitEvent(YouKicked, word, 4);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_DCCChatText(char *word[], void *userdata) {
  // $1 Address
  // $2 Port
  // $3 Nick
  // $4 The Message

  return RemoveColorAndEmitEvent(DCCChatText, word, 4);
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * La seguente call-back viene "mangiata" affinche' il client non
 * emetta sempre il messaggio di avvertimento durante l'elaborazione
 * del plug-in.
 *
 */
static int cb_ProcessAlreadyRunning(char *word[], void *userdata) {
  // A process is already running

  return XCHAT_EAT_ALL;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce la funzione cb_RemoveColors(), che viene invocata ogni
 * volta che l'utente lancia il comando: /removecolors
 *
 */  
static int cb_RemoveColors(char *word[], char *word_eol[], void *userdata) {
  if (!enable) {
    // Se il removing e' disabilitato lo abilita
    enable = 1;
    // Comunica all'utente (lo leggera' solo l'utente locale) che e'
    // stato attivato il removing.
    PluginObj.print("---\002 Colors Removing\0033 now enabled !\n");
  } else {
    // L'autoOP e' abilitato e quindi lo disabilita.
    enable = 0;
    PluginObj.print("---\002 Colors Removing\0034 now disabled !\n");
  }

  /*
   * In questo caso e' giusto "mangiare" l'evento perche' nessun'altro
   * script deve poter gestire questo comando.
   *
   */
  return XCHAT_EAT_ALL;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce la funzione di inizializzazione xchat_plugin_init().
 *
 * Si tratta della prima funzione che viene automaticamente invocata
 * all'atto del caricamento del plug-in da parte del client X-Chat.
 *
 * I Parametri passati sono rispettivamente:
 *
 * - Un puntatore al plugin in memoria (necessario per la gestione del
     plug-in stesso).
 *
 * - Un riferimento a puntatore che dovra' contenere il Nome del
     plugin (da configurare nella funzione stessa).
 *
 * - Un riferimento a puntatore che dovra' contenere la Descrizione
     del plugin (da configurare nella funzione stessa).
 *
 * - Un riferimento a puntatore che dovra' contenere la Versione del
     plugin (da configurare nella funzione stessa).
 *
 * - Gli argomenti passati ad X-Chat dalla linea di comando.
 *
 */
extern "C" {
  int xchat_plugin_init(xchat_plugin *plugin_handle,
			char **plugin_name,
			char **plugin_desc,
			char **plugin_version,
			char *arg) {
    // Configura i parametri base del plugin.
    *plugin_name = strdup("RemoveColors");
    *plugin_desc = strdup("Remove colors to message sent from remote users");
    *plugin_version = strdup("1.0.00");
    
    // Inizializza l'Oggetto plugin con i parametri base configurati.
    PluginObj = Plugin(plugin_handle, 
		       (*plugin_name), 
		       (*plugin_desc), 
		       (*plugin_version), 
		       arg);

    /*
     * Qui vengono agganciati tutti glie eventi gestiti dal client,
     * che sono anche contenuti nella lista ottenibile dalla finestra:
     * Settings->Lists->Text Events...
     *
     */    
    PluginObj.hook_print("Channel Action",			cb_ChannelAction);
    PluginObj.hook_print("Channel Action Hilight",		cb_ChannelActionHilight);
    PluginObj.hook_print("Channel Message",			cb_ChannelMessage);
    PluginObj.hook_print("Channel Msg Hilight",			cb_ChannelMsgHilight);
    PluginObj.hook_print("Channel Notice",			cb_ChannelNotice);
    PluginObj.hook_print("Kick",				cb_Kick);
    PluginObj.hook_print("Killed",				cb_Killed);
    PluginObj.hook_print("Notice",				cb_Notice);
    PluginObj.hook_print("Part with Reason",			cb_PartwithReason);
    PluginObj.hook_print("Private Message",			cb_PrivateMessage);
    PluginObj.hook_print("Private Message to Dialog",		cb_PrivateMessagetoDialog);
    PluginObj.hook_print("Quit",				cb_Quit);
    PluginObj.hook_print("Receive Wallops",			cb_ReceiveWallops);
    PluginObj.hook_print("Server Error",			cb_ServerError);
    PluginObj.hook_print("Server Notice",			cb_ServerNotice);
    PluginObj.hook_print("Server Text",				cb_ServerText);
    PluginObj.hook_print("Topic",				cb_Topic);
    PluginObj.hook_print("Topic Change",			cb_TopicChange);
    PluginObj.hook_print("WhoIs Away Line",			cb_WhoIsAwayLine);
    PluginObj.hook_print("You Kicked",				cb_YouKicked);

    // Questo evento viene agganciato per evitare che il client
    // comunichi sempre messaggio di errore durante l'elaborazione del
    // plug-in.
    PluginObj.hook_print("Process Already Running",		cb_ProcessAlreadyRunning);

    /*
     * Aggancia gli eventi "Speciali"
     *
     */
    PluginObj.hook_print("DCC Chat Text",			cb_DCCChatText);
    
    // Aggancia il comando "removecolors", in modo che ogni volta che
    // l'utente lancia suddetto venga eseguita la callback
    // cb_RemoveColors().
    PluginObj.hook_command("RemoveColors", 
			   cb_RemoveColors,
			   "Usage: REMOVECOLORS, Turns OFF/ON colors removing");
    
    /*
     * Comunica il caricamento del plug-in e quali comandi sono stati
     * aggiunti.
     *
     */    
    PluginObj.print("\002\0033RemoveColors\0036 Plugin loaded successfully !\n");
    PluginObj.printf("%s", "Command %BREMOVECOLORS%O added.\n");
    
    return 1;	/* ritorna 1 per successo */
  }
  
  /*
   * Definisce la funzione di de-inizializzazione del plug-in. Viene
   * invocata ogni volta che il modulo viene scaricato dalla memoria.
   *
   */
  int xchat_plugin_deinit(xchat_plugin *plugin_handle) {
    PluginObj.print("\002\0033RemoveColors\0036 Plugin unloaded.\n");

    return 1;       /* ritorna 1 per successo */
  }
}
